/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.components.xsql;

import oracle.xml.xsql.XSQLConnection;
import oracle.xml.xsql.XSQLConnectionManager;
import oracle.xml.xsql.XSQLConnectionManagerFactory;
import oracle.xml.xsql.XSQLPageRequest;

import javax.sql.DataSource;
import java.sql.SQLException;

/**
 * An implementation of {@link XSQLConnectionManagerFactory} which uses a {@link DataSource}
 * as its database connection pool.
 * 
 * @version $Revision: 1.1 $
 */
public class DataSourceXSQLConnectionManagerFactory implements XSQLConnectionManagerFactory {
    private DataSource dataSource;

    public XSQLConnectionManager create() {
        return new XSQLConnectionManager() {
            public XSQLConnection getConnection(String s, XSQLPageRequest xsqlPageRequest) throws SQLException {
                return new XSQLConnection(getDataSource().getConnection(), s);
            }

            public void releaseConnection(XSQLConnection xsqlConnection, XSQLPageRequest xsqlPageRequest) {
                try {
                    xsqlConnection.getJDBCConnection().close();
                }
                catch (SQLException e) {
                    throw new RuntimeException(e);
                }
            }
        };
    }

    public DataSource getDataSource() {
        return dataSource;
    }

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }
}
