/**
 * 
 * Copyright 2005 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.binding;

import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.servicemix.MessageExchangeListener;
import org.servicemix.components.util.ComponentSupport;
import org.servicemix.jbi.messaging.MessageExchangeImpl;

import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

/**
 * A base class for bindings which process inbound JBI messages
 *
 * @version $Revision: 1.8 $
 */
public abstract class OutBinding extends ComponentSupport implements Runnable, MessageExchangeListener {
    private static final Log log = LogFactory.getLog(OutBinding.class);
    private SynchronizedBoolean stop = new SynchronizedBoolean(false);
    private Thread runnable;

    public OutBinding() {
    }

    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        NormalizedMessage message = getInMessage(exchange);
        try {
            process(exchange, message);
            done(exchange);
        }
        catch (MessagingException e) {
            fail(exchange, e);
        }
    }

    /**
     * Runnable implementation
     */
    public void run() {
        try {
            DeliveryChannel deliveryChannel = getDeliveryChannel();
            while (!stop.get()) {
                MessageExchangeImpl exchange = (MessageExchangeImpl) deliveryChannel.accept();
                if (exchange != null) {
                    onMessageExchange(exchange);
                }
            }
        }
        catch (MessagingException e) {
            log.error("run failed", e);
        }
    }

    /**
     * shutdown
     *
     * @throws JBIException
     */
    public void shutDown() throws JBIException {
    }

    /**
     * stop
     *
     * @throws JBIException
     */
    public void stop() throws JBIException {
        stop.commit(true, false);
    }

    /**
     * start
     */
    public void start() throws JBIException {
        if (stop.commit(false, true)) {
            runnable = new Thread(this);
            runnable.setDaemon(true);
            runnable.start();
        }
    }

    /**
     * Process incoming
     *
     * @param messageExchange
     * @param message
     * @throws MessagingException
     */
    protected abstract void process(MessageExchange messageExchange, NormalizedMessage message) throws MessagingException;
}
