/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix;

import junit.framework.TestCase;
import org.servicemix.client.ServiceMixClient;
import org.servicemix.client.ServiceMixClientTest;
import org.servicemix.jbi.container.SpringJBIContainer;
import org.servicemix.jbi.jaxp.SourceTransformer;
import org.servicemix.jbi.jaxp.StringSource;
import org.servicemix.jbi.util.DOMUtil;
import org.servicemix.jbi.resolver.ServiceNameEndpointResolver;
import org.servicemix.examples.Receiver;
import org.servicemix.examples.MessageList;
import org.springframework.context.support.AbstractXmlApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.CachedXPathAPI;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.traversal.NodeIterator;

import javax.xml.namespace.QName;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.Fault;
import javax.jbi.JBIException;
import java.io.StringReader;
import java.io.InputStream;
import java.util.List;
import java.util.Iterator;
import java.util.Date;

/**
 * @version $Revision: 1.17 $
 */
public abstract class TestSupport extends SpringTestSupport {
    protected ServiceMixClient client;
    protected Receiver receiver;

    protected void setUp() throws Exception {
        super.setUp();

        client = (ServiceMixClient) getBean("client");

        // TODO
        //receiver = (Receiver) getBean("receiver");

        SpringJBIContainer jbi = (SpringJBIContainer) getBean("jbi");
        receiver = (Receiver) jbi.getBean("receiver");
    }

    /**
     * Sends messages to the given service and asserts that the receiver gets them all
     *
     * @param service
     * @throws javax.jbi.messaging.MessagingException
     */
    protected void assertSendAndReceiveMessages(QName service) throws Exception {
        sendMessages(service, messageCount);

        assertMessagesReceived(messageCount);
    }

    /**
     * Sends the given number of messages to the given service
     *
     * @param service
     * @throws javax.jbi.messaging.MessagingException
     */
    protected void sendMessages(QName service, int messageCount) throws Exception {
        for (int i = 1; i <= messageCount; i++) {
            InOnly exchange = client.createInOnlyExchange();

            NormalizedMessage message = exchange.getInMessage();
            message.setProperty("name", "James");
            message.setProperty("id", new Integer(i));
            message.setProperty("idText", "" + i);
            message.setContent(new StringSource("<sample id='" + i + "' sent='" + new Date() + "'>hello world!</sample>"));

            exchange.setService(service);
            client.sendSync(exchange);
            client.done(exchange);

            // lets assert that we have no failure
            Exception error = exchange.getError();
            if (error != null) {
                throw error;
            }

            Fault fault = exchange.getFault();
            assertEquals("Should have no fault!", null, fault);
        }
    }

    protected void assertMessagesReceived() throws Exception {
        assertMessagesReceived(messageCount);
    }

    protected void assertMessagesReceived(int messageCount) throws Exception {
        assertNotNull("receiver not found in JBI container", receiver);

        MessageList messageList = receiver.getMessageList();
        assertMessagesReceived(messageList, messageCount);
    }

    /**
     * Performs a request using the given file from the classpath as the request body and return the answer
     *
     * @param serviceName
     * @param fileOnClassPath
     * @return
     * @throws JBIException
     */
    protected Object requestServiceWithFileRequest(QName serviceName, String fileOnClassPath) throws JBIException {
        Source content = getSourceFromClassPath(fileOnClassPath);

        ServiceNameEndpointResolver resolver = new ServiceNameEndpointResolver(serviceName);

        Object answer = client.request(resolver, null, null, content);
        return answer;
    }

}
