/** 
 * 
 * Copyright 2005 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.examples;

import org.servicemix.components.util.PojoSupport;
import org.servicemix.MessageExchangeListener;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

/**
 * A simple POJO which just implements the {@link ComponentLifeCycle}
 * interface and is not dependent on any ServiceMix code.
 *
 * @version $Revision: 1.7 $
 */
public class ReceiverPojo extends PojoSupport implements ComponentLifeCycle, MessageExchangeListener, Receiver {

    public static final QName SERVICE = new QName("http://servicemix.org/example/", "receiver");
    public static final String ENDPOINT = "receiver";

    private ComponentContext context;
    private QName service;
    private String endpoint;
    private MessageList messageList = new MessageList();

    public ReceiverPojo() {
        this(SERVICE, ENDPOINT);
    }

    public ReceiverPojo(QName service, String endpoint) {
        this.service = service;
        this.endpoint = endpoint;
    }

    // ComponentLifeCycle interface
    //-------------------------------------------------------------------------
    public void init(ComponentContext context) throws JBIException {
        this.context = context;
        context.activateEndpoint(service, endpoint);
    }


    // MessageExchangeListener interface
    //-------------------------------------------------------------------------
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        NormalizedMessage inMessage = exchange.getMessage("in");
        messageList.addMessage(inMessage);

        exchange.setStatus(ExchangeStatus.DONE);
        context.getDeliveryChannel().send(exchange);
    }


    // Receiver interface
    //-------------------------------------------------------------------------
    public MessageList getMessageList() {
        return messageList;
    }
}
