/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.jbi.jndi;

import junit.framework.TestCase;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Hashtable;

import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @version $Revision: 1.2 $
 */
public class JNDITest extends TestCase {

    public void testSpringJNDILookup() throws Exception {
        Hashtable env = new Hashtable();
        env.put("java.naming.factory.initial", "org.servicemix.jbi.jndi.SpringInitialContextFactory");
        env.put("java.naming.provider.url", "classpath:org/servicemix/jbi/jndi/jndi.xml");
        InitialContext context = new InitialContext(env);
        assertEntryExists(context, "jdbc");
        assertEntryExists(context, "jdbc/pxe__pm");
    }

    public void testConfigureJndiInsideSpringXml() throws Exception {
        // lets load a spring context

        BeanFactory factory = new ClassPathXmlApplicationContext("org/servicemix/jbi/jndi/spring.xml");
        Object test = factory.getBean("test");
        assertNotNull("Should have found the test object", test);

        Hashtable env = new Hashtable();
        env.put("java.naming.factory.initial", "org.servicemix.jbi.jndi.SpringInitialContextFactory");
        InitialContext context = new InitialContext(env);
        assertEntryExists(context, "jdbc");
        assertEntryExists(context, "jdbc/pxe__pm");
    }
    protected void assertEntryExists(InitialContext context, String name) throws NamingException {
        Object value = context.lookup(name);
        assertNotNull(name + " should not be null", value);
    }

}
